!     ******************************************************************
      PROGRAM jscati   ! (iterative version of jscat)
!     ******************************************************************
!
!     AUTHOR: Jouni I. Peltoniemi, Max-Planck-Society, Jena, 1993, 1994 
!             pelto@georg.astro.uni-jena.de
!             tel: +49-3641-630309, fax: 55594
!
!             Some versions of the code may contain pieces stolen from
!             Bohren&Huffman, K. Muinonen, B. Michel and others.
!
!     COPYRIGHT: Fully copyrighted by the author, see file Copyright.txt for details.
!               (shortly: yet free for astrophysisists and evaluators, commercial use to be negotiated)
!
!     PURPOSE:  To compute the scattering of electromagnetic radiation
!               from an irregular, inhomogeneous particle or aggregate.
!
!     METHOD:   variational technique is used to solve the volume integral equation
!               in a least square sense using vector spherical harmonics and Bessels
!               as base functions. Integrals are approximated using dynamic rectangular grid.
!               Singularity is solved with especial rigor.
!               The normal matrix equation is solved using Lapack routine zposvx.
!
!     INPUT:  orientations; (sub)grain positions, shapes, sizes, optical properties;
!             desired accuracy, discretation parameters
!
!     OUTPUT: full phase matrix with error estimates,
!             extinction, absorption, scattering and polarization cross sections with errors
!             light curve (brightness as a function of orientation) at given direction
!             a "photograph" from given direction (under construction)
!
!     BUGS: many, but please still report.
!           Some internal bug in intrinsic matmul-routine causes segment violation error
!           with large matrices (>>1000*1000) in DEC f90 compiler.
!
!     ACCURACY: normally 0.01--10%, larger refractive indices and pushing the limits worsen,
!               depends heavily on the number of grid points and degree of expansion.
!               [proportional probably to abs(suski)/sqrt(grid points)]
!
!     LIMITATIONS:  Size parameter range: < x < 5/|m|.
!                   Shape limited by underlying grid.
!                   Sensitive to grid (non)symmetries.
!                   Extremely small or large refractive indices cause numerical difficulties.
!                   Magnetic, anisotropic, chiral, gradient index properties not fully working, yet.
!
!     DOCUMENTATION:  FastIntro.txt, Copyright.txt, warranty.txt, jscatman*.tex/.dvi/.ps/.hmtl, 
!                     jscat1.tex, and end of this file for further info.
!
!     LANGUAGE: Fortran 90, tested with NAG f90 versions 1.1, 2.0, 2.0a, 2.1, and DEC Fortran90 1.0
!               Some High Performace Features may lurk somewhere.
!
!     SUPPORT: Non-guaranteed email and telephone support available. I am also happy to come and give
!              consultation and lectures of the code, and scattering in general, if costs are covered.
!              More extensive support under consideration.
!
!     FEEDBACK: All comments and suggestions are welcome. Especially reports of (in)applicability
!               to new cases and systems. Also wishes related to user interface (input, output,
!               documentation) are desired. If the code does not do your job, please tell also
!               that; the next version may fulfil your mission, but much better if I knew what
!               feature to add next.
!
!     FUTURE: Depends a lot on Your feedpack. Plans include to generalize the code to all possible
!             linear materials (chiral, magnetic, nonisotropic ...), include  automatic
!             configuration generating and averaging sequence, have more variations in
!             output (a simulated photo is coming soon), and improve speed, accuracy, and
!             memory economy further. Iterative refinement with automatic order growth
!             is at the drawing board. 
!             It is also open whether this will remain as a free randomly developed product,
!             or be made to a fully supported commercial product. Again, that depends a lot
!             on Your wishes, ( and your $$$'s) as well as my other projects, or whether 
!             I am well enough paid for my scientific work.
!             
!
!     ACKNOWLEDGEMENTS: prof Thomas Henning, Dr Bernard Michel, Mr Ralf Stognenko, Mr Jussi Rahola,
!                       Dr Samer Kassam, Dr Karri Muinonen, Dr Francois Rouleau, Dr Peter Busseman,
!                       and many others are thanked for their help.   
!
!
!     by  Jouni I. Peltoniemi, 
!                              pelto@georg.astro.uni-jena.de
!     ------------------------------------------------------------------

!  The main program contains the averaging and refining sequence, 
!  all the rest is in modules. Calls do have controlled side effects.


	 USE constants                                            
! including pi,sqrt(-1),etc
	 USE timing,ONLY:CPUtime                                  
	 USE vectorix,ONLY:                                       
! some  vector operations and types
	 USE scatterer,ONLY:InitScatterer,InitEnsamble,OriAveType 
! scatterer data (shape, optical properties)
	 USE incidentia,ONLY:incidences,getwavelength             
! incident fields 
	 USE defbasegrid,ONLY:DefineBaseGrid                      
! discretization details
	 USE fillmatrices,ONLY:FillTheMatrices
	 USE resultia,ONLY:InitDetectors,StoreResults,FinalOutput 
! for final outputs (independent of discretization)
	 USE matrixia,ONLY:Solve                                  
! matrix equation solver
	 USE special,ONLY:                                        
! Bessels and spherical harmonics etc
	 USE mie,ONLY:


!  -------------------------------------------------------------------------

	 IMPLICIT NONE

!                          ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	 CHARACTER(22),PARAMETER::   Version='0.60, ALPHA, 3. Nov. 1994'      !
	!                          ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~



	 logical ::resultsOK=.TRUE.
	 integer ::dim,cdim
	 integer ::Orientations=1    
	 integer ::Configurations=1,conf
	 integer,parameter::polarizations=2

	  complex*16, allocatable::e(:,:,:)  
! the unknown expansion coefficients; to be solved later
	  complex*16, allocatable::A(:,:)    
! the coefficient matrix; to be evaluated here
	  complex*16, allocatable::e0(:,:,:) 
! the righthandside; to be evaluated here
	  complex*16, SAVE,DIMENSION(:,:,:,:,:,:,:),
     &     ALLOCATABLE::Residual

	  CALL Greetings(Version)
	  CALL FastIntro
	  CALL Copyright  

	  CALL GetWavelength
	  CALL InitEnsamble(Configurations,Orientations,OriAveType) 
	  IF (CPUtime()>22.0_singl) write(*,*)
     &    'CPU time after initializing scatterer ensamble',CPUtime()
	  call Incidences(Orientations,OriAveType)
	  CALL InitDetectors
	  averaging: DO conf=1,Configurations
	     call InitScatterer(conf)
	     CALL DefineBaseGrid(dim,cdim)
           write(*,*)'CPU time after initializing the grid',CPUtime()
	     Residual=0.0
	     iteration: DO
	       ALLOCATE(A(dim,dim),e0(dim,orientations,polarizations))        
	       CALL FillTheMatrices(A,e0,Residual,dim,cdim,
     &               orientations,polarizations)
                write(*,*)'CPU time after filling the matrices',CPUtime()
	       ALLOCATE(e(dim,orientations,polarizations))        
	       CALL Solve(A,e,e0,dim,Orientations*polarizations,
     &                              method='ZPOSVX')  !e <- A*e=e0
	       write(*,*)'CPU time after solving the normal equation',CPUtime()
	       DEALLOCATE(A,e0)
	       CALL GetResiduals(e,Residual,resultsOK)
	       IF (ResultsOK) EXIT iteration
	       CALL NewBasis(dim,cdim,Residual)
	     ENDDO iteration
	!   write(*,*)'e1',e(:,:,1)
	!   write(*,*)'e2',e(:,:,2)
	       CALL StoreResults(conf,Configurations,dim,Orientations,
     &                           polarizations,e)
	       DEALLOCATE(e) !; CALL FreeGrid
	       write(*,*)'CPU time after computing and storing results',
     &                CPUtime()
	    END DO averaging 
	    CALL FinalOutput(orientations*configurations)
	   END PROGRAM Jscat
	! ******************************************************************




	SUBROUTINE Greetings(Version)
	 CHARACTER(*),INTENT(in)::Version
	 INTEGER::timevalue(8)

	  write (*,*)
	  write (*,*)'---------------------------------------------------'
	  write (*,*)
	  CALL DATE_AND_TIME(values=timevalue)
	  SELECT CASE(timevalue(5))
	    CASE(5:11); write (*,*) 'Good morning!'
	    CASE(12:17); write (*,*) 'Good afternoon!'
	    CASE(18:22); write (*,*) 'Good evening!'
	    CASE DEFAULT; write (*,*) 'Good night!'
	  END SELECT
	  write (*,*)
	  write (*,*)  'This is jscat version ',VERSION,' on Your service.'
	  write (*,*)'(A,I2,A,I2,A,I5,A,I2,A,I2,A)')  &
	   'Program started ',timevalue(3),'.',timevalue(2),'.',
     &       timevalue(1),' at ',timevalue(5),':',timevalue(6),' o''clock.'
	  write (*,*)'---------------------------------------------------'
	END



	SUBROUTINE FastIntro

	CHARACTER(80) Line
	  OPEN(3,status='OLD',FILE='FastIntro.txt',ERR=233)
	  DO
	    READ (3,*,END=231,ERR=233) Line
	    WRITE(*,*) Line
	  ENDDO
 231      CLOSE(3)
	  RETURN  
 233      write (*,*)
	  write (*,*)  'jscat solves the scattering of '
          write (*,*)  'electromagnetic radiation from'
	  write (*,*)  'an irregular particle or aggregate '
          write (*,*)  'by discretizing the'
	  write (*,*)  'Volume Integral Equation and '
          write (*,*)  'minimizing the Lagrangian.'
	  write (*,*)
	END

	SUBROUTINE Copyright

	 CHARACTER(80) Line
	  OPEN(3,status='OLD',FILE='Copyright.txt',ERR=433)
	  DO
	    READ (3,*,END=431,ERR=433) Line
	    WRITE(*,*) Line
	  ENDDO
 431       CLOSE(3)
	  RETURN  
	     write (*,*)  'by Jouni I. Peltoniemi, 1993-94, Jena,'
 433         write (*,*)  '  pelto@georg.astro.uni-jena.de'
	     write (*,*)  '======================================='
 END 





!     Variational technique is used to convert the integral equation of em-radiation
!
!        E(r)=E0(r)+\int dr' Green(r,r')*(m**2-1)E(r')                             (1)
!  ,or
!        |E_0>=|E>-GV|E>                                                           (1b)
!
!
!     to numerically tractable form (i.e. discretized).
!
!     The scatterer is divided to a finite number of homogeneous cells
!     and the fields inside the cells are approximated by a finite order expansion
!     of basis functions |i>:
!                             |E>=sum_i |i><i|E>                                    (2)
!                          
!    Here, spherical waves are used as basefunctions.     
!
!    Due to non-completeness of the expansion, Eq. 1 cannot be 
!    rigorously fulfilled. We define a residual R as
!
!        R=|E_0>-|E>+GV|E> ,                                                        (3)
! 
!    and look for solutions minimizing its square norm, yielding normal equation:
!
!        <w_j|E_0>=sum_i [ <w_j|i>-<w_j|GV|i> ] <i|E>                              (4)
!
!    for the N unknown coefficients <i|E>=e_i. 
!    Same in a matrix form:
!
!         A*e=e0.                                                                   (4b)
!
!     If the solution with the first base function set is not satisfactory,
!     one can iteratively introduce higher and higher order terms where |R| is too large.
!     The equation for the new unknowns read:
!      
!        <w_j|R>=sum_i [ <w_j|i>-<w_j|GV|i> ] <i|E> ,                             (5)
!
!     where R is the residual with the old set.
!     Later, one computes the cross-sections and the scattering matrix from the internal field.


!     This code provides two averaging possibilities:
!      1. Orientation averaging (same particle same wavelength)
!      2. Configuration averaging (over different particles or wavelengths) [to be done]
!     Arbitrary distributions can be used (in future, now only uniform random).

!     Outputs are various mean cross-sections and the mean scattering phase matrix.


!     As a limiting case the field scattered by one fixed particle can be computed
!     in a finite number of arbitrary locations.

